#include "F_Click_Menu_Code.dm"


/********************************************************************

	F_Click_Menu allows you to easily add clickable menus to your game
	or project.  You may set up the color of the menu text, background
	and the 'mouse_over' color.  Currently, the menus are static, and
	the same for everybody.  Its also up to you to keep the client screen
	large enough for the menu to remain visible.  See below for an
	explanation of how to set up and use the library.

	While not overly feature rich, this library is very simple to use
	and configure.

                                                     flick()

 ********************************************************************/


/*
	There are only a couple of procs required to use the F_Click_Menu
	system.  F_Click_Menu.cmenu_Show(client/C) and F_Click_Menu.cmenu_Hide(client/C).

*/

f_cmenu_control/cmenu_Show(client/C)
		/*
			Shows the menu to the specified client.  To use, call F_Click_Menu.cmenu_Show(C)
			where C is a client
		*/
	return ..()



f_cmenu_control/cmenu_Hide(client/C)
		/*
			Hides the menu from the specified client.  To use, call F_Click_Menu.cmenu_Hide(C)
			where C is a client
		*/
	return ..()



/*
	  To set up your menu system, simply add new subtypes beneath the f_click_menu type.
		Example:


		f_click_menu
			Test							//  This will be part of the main menu bar
				menu1						//  menu1 and menu2 will be Test's menu that appears when you
				menu2						// 	click Test

			Help							//	part of the main menu bar...
				About						// 	Help's menu...
				Help_On
					Broken 				//	This won't show up.  Only two layers are supported


		Only the first two levels of subtypes will be used.  The direct subtypes to
		f_click_menu become the menu bar ('Test' and 'Help' in this case), and the subtypes
		of those become menu elements	of each main menu section.

		Each of the secondary subtypes contains a proc called 'cmenu_Clicked_By(client/C)'
		Whenever one of them is clicked, the client who clicked it is sent as an argument
		to this proc.  You can than do whatever you want from their.  (ie. Show them a help
		file, enter some information, file a bug report... whatever.  See the demo for examples.)

		Example:

		f_click_menu
			Test
				menu1
					Clicked_By(client/C)
						C << "You clicked menu option 'Test/menu1'"		//Send them a message

				menu2
					Clicked_By(client/C)
						C.mob.Move(locate(1,1,1))				//	You can also affect the client.mob

			Help
				About
					Clicked_By(client/C)
						C << "F_Click_Menu:  version 1"

				Help_On
					Clicked_By(client/C)
						var/HTML = "<B>Help On</B><BR><BR>This would be a help file, if there was one."
						C << browse(HTML)


		Thats it.  You now have a working menu system.

*/

/*
		There are three color values you can change if you would like.  They effect the menu
		background color, menu text color and the highlighted menu text color.  Simply change
		their value to whatever hex based color you would like.  They can only be set at
		compile time.
*/

f_cmenu_control
	cmenu_text_color = "#000000"
	cmenu_highlight_color = "#0000ff"
	cmenu_background_color = "#bbbbbb"
