/**********************************************************************

	F_Click_Menu allows you to easily add clickable menus to your game
	or project.  You may set up the color of the menu text, background
	and the 'mouse_over' color.  Currently, the menus are static, and
	the same for everybody.  Its also up to you to keep the client screen
	large enough for the menu to remain visible.  See F_Click_Menu.dm for
	an explanation of how to set up and use the library.

                                                   flick()

**********************************************************************/


/**********************************************************************
*                                                                     *
*  This is the code section of the library.  You DON'T need to do     *
*  anything to this to get the library to work.  Read F_Click_Menu.dm *
*  for instructions.                                                  *
*                                                                     *
**********************************************************************/



//		This is the basic menu object.  There is only one type, though
//		though they perform two different actions, depending on whether
//		they are primary menus or not.  Direct subtypes of f_click_menu
//		act as the main menu topics.  Subtypes of the topics act as menu
//		selections for that topic.


f_click_menu
	var
		cmenu_children[0]
		cmenu_primary = FALSE
		f_click_menu/cmenu_Owner
		image/cmenu_highlight_image

	parent_type = /obj
	layer = 1001


//		If this is a primary menu, either open or close its submenu depending
//		on its current state.  If a secondary menu, close the submenu, and call
//		the secondary menu's cmenu_Clicked_By() proc

	Click()
		if(cmenu_primary)
			if(usr.client.f_cmenu_looking)
				F_Click_Menu.cmenu_Hide_Menu(usr.client, src)
				usr.client.f_cmenu_looking = FALSE
			else
				F_Click_Menu.cmenu_Set_Menu(usr.client, src)
				usr.client.f_cmenu_looking = TRUE
		else
			F_Click_Menu.cmenu_Hide_Menu(usr.client, cmenu_Owner)
			usr.client.f_cmenu_looking = FALSE
			cmenu_Clicked_By(usr.client)


//		Show the menu's highlight image to the usr

	MouseEntered()
		usr.client.images += cmenu_highlight_image
		if(cmenu_primary && usr.client.f_cmenu_looking)
			F_Click_Menu.cmenu_Set_Menu(usr.client, src)
		..()


//		Hide the menu's highlight image

	MouseExited()
		usr.client.images -= cmenu_highlight_image
		..()

	proc

//		This is the proc which is called whenever a client clicks on a secondary menu.
//		When defining your menu system, you can put whatever you want in this proc.

		cmenu_Clicked_By(client/C)


//		Set the overlays for each menu element and its highlight image.  The yoffset var
// 		determines how high up the menu list the element is, and the maxlength var is
//		the length of the longest element in the current menu.

		cmenu_Set_Overlays(yoffset = 0, maxlength)
			var/obj/cmenu_Highlight = new()
			var/tmpname = " [name]"
			if(!maxlength)
				maxlength = length(tmpname)
			var/i
			for(i in 1 to maxlength)
				var/letter
				if(i > length(tmpname))
					letter = " "
				else
					letter = copytext(tmpname, i, i+1)
				var/obj/cmenu_letter/L = new(src,letter , i-1, src, yoffset)
				overlays += L
				var /obj/cmenu_letter/highlight/H = new(src,letter , i-1, src, yoffset)
				cmenu_Highlight.overlays += H
				cmenu_highlight_image = image(cmenu_Highlight, src)
				del H
				del L
			del cmenu_Highlight
			return length(tmpname)-1



//		Initialize a main menu element.  Offset determines how far over to place the
//		menu element.  For each menu element within the main element, call the child's
//		cmenu_Initialize_Child() proc. Returns a list of child elements and the width
//		of the main element to the calling F_Click_Menu.F_Cmenu_Set_Menu() proc.

		cmenu_Initialize_Main(offset)
			cmenu_primary = TRUE
			var/X = round(offset*8/32)
			pixel_x = (offset*8)%32
			screen_loc = "[X+1],1"
			var/counter = 1
			var/maxlen
			var/tempchildren[0]
			for(var/V in typesof(type)-type)
				var/f_click_menu/child = new V(null, TRUE)
				if(child.parent_type == type)
					maxlen = max(maxlen, length(child.name))
				tempchildren += child
			for(var/f_click_menu/child in tempchildren)
				if(child.parent_type == type)
					cmenu_children += child
					child.cmenu_Initialize_Child(src, counter, maxlen+1)
					counter++
			var/width = cmenu_Set_Overlays(,length("[name]")+2)
			var/L[0]
			L["width"] = width
			L["children"] = cmenu_children
			return L



//		Initialize a child menu element.  Y is the position of the menu element within
//		the owner menu and length is the length of the longest element within the same menu.

		cmenu_Initialize_Child(f_click_menu/owner,  Y,length)
			cmenu_Owner = owner
			screen_loc = cmenu_Owner.screen_loc
			pixel_x = cmenu_Owner.pixel_x + 8
			pixel_y = Y*13
			cmenu_Set_Overlays(pixel_y, length)



//		Only called for primary menus.  Hides the children from the client.

		cmenu_Hide(client/C)
			C.screen -= cmenu_children



//		Only called for primary menus.  Shows the children to the client.

		cmenu_Show(client/C)
			C.f_cmenu_current = src
			C.screen += cmenu_children



//		This is the letter object used to make the menu text overlays.  loc is the
//		menu object, Letter determines which icon_state to use, xoffset determines
//		the position of the letter in the word and yoffset determines the position
//		of the word in the menu.

obj
	cmenu_letter

		layer = 1001

		New(loc, Letter, xoffset, f_click_menu/owner, yoffset = 0)
			if(isobj(loc))
				var/obj/Owner = loc
				icon = F_Click_Menu.cmenu_TextIcon()		// Uses the normal text icon
				underlays += F_Click_Menu.cmenu_Background()
				if(Letter)
					cmenu_Set_State("[Letter]")
				pixel_x = 8*xoffset + Owner.pixel_x
				pixel_y = yoffset
			..()

		proc
			cmenu_Set_State(Letter)
				icon_state = Letter

		highlight
			New()
				..()
				icon = F_Click_Menu.cmenu_HighlightIcon()	//	Uses the highlighted text icon



var/f_cmenu_control/F_Click_Menu

world
	New()
		if(!F_Click_Menu)
			F_Click_Menu = new()
		..()



//		This is the menu control datum.  The above F_Click_Menu var will be created
//		in world/New(), and will be used to control the menu/client input and output.
//		The menu control is what actually cycles though the f_click_menu types and
//		seperates them into 'categories', and 'secondaries'.  Categories are the main
//		menu selections, and secondaries are the menu choices of each category.  The
//		three icons and corresponding colors are created and used to create the various
//		menu, highlight and background elements of the menu system.

f_cmenu_control
	var
		cmenu_categories[0]
		cmenu_secondaries[0]
		icon/cmenu_Ihighlight
		icon/cmenu_Itext_color
		icon/cmenu_Ibackground_color
		cmenu_text_color = "#000000"           /*  _     These three vars may be changed to  */
		cmenu_highlight_color = "#0000ff"      /*   \    adjust the menu text, highlight     */
		cmenu_background_color = "#bbbbbb"     /*  _/    and background colors.              */
		                                       /*        See F_Click_Menu.dm for help        */
	New()
		..()
		spawn()
			F_CMenu_Set_Menu()


	proc

//		Set up the three icons, and then begin setting up menu items.  For menu items
//		whose parent type is /f_click/menu, call cmenu_Initialize_Main(width), where width
//		is the total width of the main menu bar so far.

		F_CMenu_Set_Menu()
			cmenu_Ihighlight = new('cmenu_letters.dmi')
			cmenu_Ihighlight.SwapColor("#000000", cmenu_highlight_color)
			cmenu_Itext_color = new('cmenu_letters.dmi')
			cmenu_Itext_color.SwapColor("#000000", cmenu_text_color)
			cmenu_Ibackground_color = new('cmenu_underlay.dmi')
			cmenu_Ibackground_color.SwapColor("#FFFFFF", cmenu_background_color)
			var/width = 0
			for(var/V in typesof(/f_click_menu) - /f_click_menu)
				var/f_click_menu/F = new V(null, TRUE)
				if(F.parent_type == /f_click_menu)
					cmenu_categories += F
					var/list/Params = F.cmenu_Initialize_Main(width)
					width += Params["width"] + 2
					cmenu_secondaries["[F.name]"] = Params["children"]
				else
					del F

//		The next two procs are the only ones you need to call to use the menu system.
//		They show or hide the entire menu from the client.

//		This shows the main menu to the client.

		cmenu_Show(client/C)
			for(var/f_click_menu/F in cmenu_categories)
				C.screen += F


//		This hides the main menu from the client.
		cmenu_Hide(client/C)
			for(var/f_click_menu/F in C.screen)
				C.screen -= F



//		The next three procs return the appropriate icon

		cmenu_HighlightIcon()
			return cmenu_Ihighlight

		cmenu_TextIcon()
			return cmenu_Itext_color

		cmenu_Background()
			return cmenu_Ibackground_color



//		These procs are used internally to show or hide the submenus of each main
//		menu element (or category)

		cmenu_Set_Menu(client/C, f_click_menu/F)
			if(C.f_cmenu_current)
				C.f_cmenu_current.cmenu_Hide(C)
			F.cmenu_Show(C)

		cmenu_Hide_Menu(client/C, f_click_menu/F)
			C.f_cmenu_current = null
			F.cmenu_Hide(C)


client
	var
		f_click_menu/f_cmenu_current  //	Which submenu, if any is the client currently showing
		f_cmenu_looking = FALSE       //	Is the client looking at a submenu

	New()
		..()
		spawn()
			if(!F_Click_Menu)
				F_Click_Menu = new()


//		Check to see if the client has a menu open, and clicked something else.  If so, close
//		the menu and change the f_cmenu_looking var to FALSE

	Click(Object)
		if(!istype(Object, /f_click_menu) && f_cmenu_looking)
			F_Click_Menu.cmenu_Hide_Menu(src, f_cmenu_current)
			f_cmenu_looking = FALSE
		..()
